@@ -6,30 +6,16 @@ from Cryptodome.Cipher import AES
 
 from datetime import datetime
 
+from .helpers import (
+    to_mac,
+    to_unformatted_mac,
+)
+
 _LOGGER = logging.getLogger(__name__)
 
 # Device type dictionary
 # {device type code: device name}
 XIAOMI_TYPE_DICT = {
-    0x01AA: "LYWSDCGQ",
-    0x045B: "LYWSD02",
-    0x055B: "LYWSD03MMC",
-    0x1203: "XMWSDJ04MMC",
-    0x04E1: "XMMF01JQD",
-    0x0098: "HHCCJCY01",
-    0x03BC: "GCLS002",
-    0x015D: "HHCCPOT002",
-    0x040A: "WX08ZM",
-    0x098B: "MCCGQ02HL",
-    0x0083: "YM-K1501",
-    0x0113: "YM-K1501EU",
-    0x045C: "V-SK152",
-    0x0863: "SJWS01LM",
-    0x07F6: "MJYD02YL",
-    0x03DD: "MUE4094RT",
-    0x0A8D: "RTCGQ02LM",
-    0x00DB: "MMC-T201-1",
-    0x0489: "M1S-T500",
     0x0C3C: "CGC1",
     0x0576: "CGD1",
     0x066F: "CGDK2",
@@ -37,23 +23,52 @@ XIAOMI_TYPE_DICT = {
     0x0B48: "CGG1-ENCRYPTED",
     0x03D6: "CGH1",
     0x0A83: "CGPR1",
-    0x06d3: "MHO-C303",
-    0x0387: "MHO-C401",
+    0x03BC: "GCLS002",
+    0x0098: "HHCCJCY01",
+    0x015D: "HHCCPOT002",
     0x02DF: "JQJCY01YM",
     0x0997: "JTYJGD03MI",
     0x1568: "K9B-1BTN",
     0x1569: "K9B-2BTN",
     0x0DFD: "K9B-3BTN",
+    0x1889: "MS1BB(MI)",
+    0x2AEB: "HS1BB(MI)",
+    0x01AA: "LYWSDCGQ",
+    0x045B: "LYWSD02",
+    0x16e4: "LYWSD02MMC",
+    0x055B: "LYWSD03MMC",
+    0x098B: "MCCGQ02HL",
+    0x06d3: "MHO-C303",
+    0x0387: "MHO-C401",
+    0x07F6: "MJYD02YL",
+    0x04E9: "MJZNMSQ01YD",
+    0x00DB: "MMC-T201-1",
+    0x03DD: "MUE4094RT",
+    0x0489: "M1S-T500",
+    0x0A8D: "RTCGQ02LM",
+    0x0863: "SJWS01LM",
+    0x045C: "V-SK152",
+    0x040A: "WX08ZM",
+    0x04E1: "XMMF01JQD",
+    0x1203: "XMWSDJ04MMC",
+    0x1949: "XMWXKG01YL",
+    0x098C: "XMZNMST02YD",
+    0x0784: "XMZNMS04LM",
+    0x0E39: "XMZNMS08LM",
     0x07BF: "YLAI003",
     0x0153: "YLYK01YL",
     0x068E: "YLYK01YL-FANCL",
     0x04E6: "YLYK01YL-VENFAN",
     0x03BF: "YLYB01YL-BHFRC",
     0x03B6: "YLKG07YL/YLKG08YL",
+    0x0083: "YM-K1501",
+    0x0113: "YM-K1501EU",
     0x069E: "ZNMS16LM",
     0x069F: "ZNMS17LM",
-    0x04E9: "MJZNMSQ01YD",
-    0x098C: "XMZNMST02YD",
+    0x0380: "DSL-C08",
+    0x0DE7: "SU001-T",
+    0x20DB: "MJZNZ018H",
+    0x18E3: "ZX1",
 }
 
 # Structured objects for data conversions
@@ -128,7 +143,7 @@ BLE_LOCK_METHOD = {
 
 
 # Advertisement conversion of measurement data
-# https://iot.mi.com/new/doc/embedded-development/ble/object-definition
+# https://iot.mi.com/new/doc/accesses/direct-access/embedded-development/ble/object-definition
 def obj0003(xobj):
     """Motion"""
     return {"motion": xobj[0], "motion timer": xobj[0]}
@@ -199,12 +214,41 @@ def obj0007(xobj):
     return {"door": door, "door action": action}
 
 
+def obj0008(xobj, device_type):
+    """armed away"""
+    returnData = {}
+    value = xobj[0] ^ 1
+    returnData.update({'armed away': value})
+    if len(xobj) == 5:
+        timestamp = int.from_bytes(xobj[1:], 'little')
+        timestamp = datetime.fromtimestamp(timestamp).isoformat()
+        returnData.update({'timestamp': timestamp})
+    # Lift up door handle outside the door sends this event from DSL-C08.
+    if device_type == "DSL-C08":
+        return{
+            "lock": value,
+            "locktype": 'lock',
+            "action": 'lock outside the door',
+            "method": "manual",
+            "error": None,
+            "key id": None,
+            "timestamp": None,
+        }
+    return returnData
+
+
 def obj0010(xobj):
     """Toothbrush"""
     if xobj[0] == 0:
-        return {'toothbrush': 1, 'counter': xobj[1]}
+        if len(xobj) == 1:
+            return {'toothbrush': 1}
+        else:
+            return {'toothbrush': 1, 'counter': xobj[1]}
     else:
-        return {'toothbrush': 0, 'score': xobj[1]}
+        if len(xobj) == 1:
+            return {'toothbrush': 0}
+        else:
+            return {'toothbrush': 0, 'score': xobj[1]}
 
 
 def obj000b(xobj, device_type):
@@ -235,6 +279,12 @@ def obj000b(xobj, device_type):
         action = BLE_LOCK_ACTION[action][2]
         method = BLE_LOCK_METHOD[method]
 
+        # Biometric unlock then disarm
+        if device_type == "DSL-C08":
+            if method == "password":
+                if 5000 <= key_id < 6000:
+                    method = "one-time password"
+
         return {
             lock_type: lock,
             "locktype": lock_type,
@@ -415,21 +465,21 @@ def obj1001(xobj, device_type):
             result["button"] = button_press_type
         elif device_type == "K9B-1BTN":
             result["button switch"] = btn_switch_press_type
-            result["1_btn_switch"] = one_btn_switch
+            result["one btn switch"] = one_btn_switch
         elif device_type == "K9B-2BTN":
             result["button switch"] = btn_switch_press_type
             if two_btn_switch_left:
-                result["2_btn_switch_left"] = two_btn_switch_left
+                result["two btn switch left"] = two_btn_switch_left
             if two_btn_switch_right:
-                result["2_btn_switch_right"] = two_btn_switch_right
+                result["two btn switch right"] = two_btn_switch_right
         elif device_type == "K9B-3BTN":
             result["button switch"] = btn_switch_press_type
             if three_btn_switch_left:
-                result["3_btn_switch_left"] = three_btn_switch_left
+                result["three btn switch left"] = three_btn_switch_left
             if three_btn_switch_middle:
-                result["3_btn_switch_middle"] = three_btn_switch_middle
+                result["three btn switch middle"] = three_btn_switch_middle
             if three_btn_switch_right:
-                result["3_btn_switch_right"] = three_btn_switch_right
+                result["three btn switch right"] = three_btn_switch_right
         else:
             return None
         return result
@@ -516,10 +566,10 @@ def obj1015(xobj):
 def obj1017(xobj):
     """Motion"""
     if len(xobj) == 4:
-        (motion,) = M_STRUCT.unpack(xobj)
+        (no_motion_time,) = M_STRUCT.unpack(xobj)
         # seconds since last motion detected message (not used, we use motion timer in obj000f)
         # 0 = motion detected
-        return {"motion": 1 if motion == 0 else 0}
+        return {"motion": 1 if no_motion_time == 0 else 0, "no motion time": no_motion_time}
     else:
         return {}
 
@@ -566,6 +616,18 @@ def obj100d(xobj):
         return {}
 
 
+def obj100e(xobj, device_type):
+    """Lock common attribute"""
+    # https://iot.mi.com/new/doc/accesses/direct-access/embedded-development/ble/object-definition#%E9%94%81%E5%B1%9E%E6%80%A7
+    if len(xobj) == 1:
+        # Unlock by type on some devices
+        if device_type == "DSL-C08":
+            lock_attribute = int.from_bytes(xobj, 'little')
+            lock = lock_attribute & 0x01 ^ 1
+            childlock = lock_attribute >> 3 ^ 1
+            return {"childlock": childlock, "lock": lock}
+
+
 def obj2000(xobj):
     """Body temperature"""
     if len(xobj) == 5:
@@ -582,7 +644,8 @@ def obj2000(xobj):
         return {}
 
 
-# The following data objects are device specific. For now only added for XMWSDJ04MMC
+# The following data objects are device specific. For now only added for
+# LYWSD02MMC, XMWSDJ04MMC, XMWXKG01YL, LINPTECH MS1BB(MI), HS1BB(MI)
 # https://miot-spec.org/miot-spec-v2/instances?status=all
 def obj4803(xobj):
     """Battery"""
@@ -590,6 +653,119 @@ def obj4803(xobj):
     return {"battery": batt}
 
 
+def obj4804(xobj):
+    """Opening status"""
+    opening_state = xobj[0]
+    # Status of the door/window, used in combination with obj4a12
+    if opening_state == 1:
+        opening = 1
+    elif opening_state == 2:
+        opening = 0
+    else:
+        return {}
+    return {"opening": opening}
+
+
+def obj4805(xobj):
+    """Illuminance in lux"""
+    (illu,) = struct.unpack("f", xobj)
+    return {"illuminance": illu}
+
+
+def obj4810(xobj):
+    """Sleep State"""
+    sleep_state = xobj[0]
+    if sleep_state == 0:
+        return {"sleeping": 0}
+    elif sleep_state == 1:
+        return {"sleeping": 1}
+    elif sleep_state == 2:
+        return{"button switch": "double press"}
+    else:
+        return None
+
+
+def obj4811(xobj):
+    """Snoring State"""
+    return {"snoring": xobj[0]}
+
+
+def obj4818(xobj):
+    """Time in seconds of no motion (not used, we use 4a08)"""
+    if len(xobj) == 2:
+        (no_motion_time,) = struct.unpack("<H", xobj)
+        # seconds since last motion detected message (not used, we use motion timer in obj4a08)
+        # 0 = motion detected
+        return {"motion": 1 if no_motion_time == 0 else 0, "no motion time": no_motion_time}
+    else:
+        return {}
+
+
+def obj4a01(xobj):
+    """Low Battery"""
+    low_batt = xobj[0]
+    return {"low battery": low_batt}
+
+
+def obj4a08(xobj):
+    """Motion detected with Illuminance in lux"""
+    (illu,) = struct.unpack("f", xobj)
+    return {"motion": 1, "motion timer": 1, "illuminance": illu}
+
+
+def obj4a0f(xobj):
+    """Door/window broken open"""
+    dev_forced = xobj[0]
+    if dev_forced == 1:
+        return {
+            "opening": 1,
+            "status": "door/window broken open"
+        }
+    else:
+        return {}
+
+
+def obj4a12(xobj):
+    """Opening event"""
+    opening_state = xobj[0]
+    # Opening event, used in combination with obj4804
+    if opening_state == 1:
+        opening = 1
+    elif opening_state == 2:
+        opening = 0
+    else:
+        return {}
+    return {"opening": opening}
+
+
+def obj4a13(xobj):
+    """Button"""
+    click = xobj[0]
+    if click == 1:
+        return {"button": "toggle"}
+    else:
+        return {}
+
+
+def obj4a1a(xobj):
+    """Door Not Closed"""
+    if xobj[0] == 1:
+        return {
+            "opening": 1,
+            "status": "door not closed"}
+    else:
+        return {}
+
+
+def obj4c02(xobj):
+    """Humidity"""
+    if len(xobj) == 1:
+        humi = xobj[0]
+        return {"humidity": humi}
+    else:
+        return {}
+
+
 def obj4c01(xobj):
     """Temperature"""
     if len(xobj) == 4:
@@ -599,6 +775,12 @@ def obj4c01(xobj):
         return {}
 
 
+def obj4c03(xobj):
+    """Battery"""
+    batt = xobj[0]
+    return {"battery": batt}
+
+
 def obj4c08(xobj):
     """Humidity"""
     if len(xobj) == 4:
@@ -608,12 +790,145 @@ def obj4c08(xobj):
         return {}
 
 
+def obj4c14(xobj):
+    """Mode"""
+    mode = xobj[0]
+    return {"mode": mode}
+
+
+def obj4e0c(xobj):
+    """Click"""
+    click = xobj[0]
+    btn_switch_press_type = "single press"
+    two_btn_switch_left = None
+    two_btn_switch_right = None
+    if click == 1:
+        two_btn_switch_left = "toggle"
+    elif click == 2:
+        two_btn_switch_right = "toggle"
+    elif click == 3:
+        two_btn_switch_left = "toggle"
+        two_btn_switch_right = "toggle"
+    else:
+        btn_switch_press_type = None
+    return {
+        "two btn switch left": two_btn_switch_left,
+        "two btn switch right": two_btn_switch_right,
+        "button switch": btn_switch_press_type,
+    }
+
+
+def obj4e0d(xobj):
+    """Double Click"""
+    click = xobj[0]
+    btn_switch_press_type = "double press"
+    two_btn_switch_left = None
+    two_btn_switch_right = None
+    if click == 1:
+        two_btn_switch_left = "toggle"
+    elif click == 2:
+        two_btn_switch_right = "toggle"
+    elif click == 3:
+        two_btn_switch_left = "toggle"
+        two_btn_switch_right = "toggle"
+    else:
+        btn_switch_press_type = None
+    return {
+        "two btn switch left": two_btn_switch_left,
+        "two btn switch right": two_btn_switch_right,
+        "button switch": btn_switch_press_type,
+    }
+
+
+def obj4e0e(xobj):
+    """Long Press"""
+    click = xobj[0]
+    btn_switch_press_type = "long press"
+    two_btn_switch_left = None
+    two_btn_switch_right = None
+    if click == 1:
+        two_btn_switch_left = "toggle"
+    elif click == 2:
+        two_btn_switch_right = "toggle"
+    elif click == 3:
+        two_btn_switch_left = "toggle"
+        two_btn_switch_right = "toggle"
+    else:
+        btn_switch_press_type = None
+    return {
+        "two btn switch left": two_btn_switch_left,
+        "two btn switch right": two_btn_switch_right,
+        "button switch": btn_switch_press_type,
+    }
+
+
+def obj4e16(xobj):
+    """Bed occupancy"""
+    event = xobj[0]
+    if event == 1:
+        return {"bed occupancy": 1}
+    else:
+        return None
+
+
+def obj4e17(xobj):
+    """Bed occupancy"""
+    event = xobj[0]
+    if event == 1:
+        return {"bed occupancy": 0}
+    else:
+        return None
+
+
+def obj5010(xobj):
+    """Sleep State"""
+    sleep_state = xobj[0]
+    if sleep_state == 0:
+        return {"sleeping": 0}
+    elif sleep_state == 1:
+        return {"sleeping": 1}
+    elif sleep_state == 2:
+        return{"button": "double press"}
+    else:
+        return None
+
+
+def obj5011(xobj):
+    """Snoring State"""
+    return {"snoring": xobj[0]}
+
+
+def obj5403(xobj):
+    """Battery level"""
+    return {"battery": xobj[0]}
+
+
+def obj5601(xobj):
+    """Low Battery"""
+    low_batt = xobj[0]
+    return {"low battery": low_batt}
+
+
+def obj5a16(xobj):
+    """Bed occupancy"""
+    event = xobj[0]
+    if event == 1:
+        return {"bed occupancy": 1}
+    elif event == 2:
+        return {"bed occupancy": 0}
+    elif event == 3:
+        return{"button": "double press"}
+    else:
+        return None
+
+
 # Dataobject dictionary
 # {dataObject_id: (converter}
 xiaomi_dataobject_dict = {
     0x0003: obj0003,
     0x0006: obj0006,
     0x0007: obj0007,
+    0x0008: obj0008,
     0x0010: obj0010,
     0x000B: obj000b,
     0x000F: obj000f,
@@ -634,10 +949,35 @@ xiaomi_dataobject_dict = {
     0x1019: obj1019,
     0x100A: obj100a,
     0x100D: obj100d,
+    0x100E: obj100e,
     0x2000: obj2000,
     0x4803: obj4803,
+    0x4804: obj4804,
+    0x4805: obj4805,
+    0x4810: obj4810,
+    0x4811: obj4811,
+    0x4818: obj4818,
+    0x4a01: obj4a01,
+    0x4a08: obj4a08,
+    0x4a0f: obj4a0f,
+    0x4a12: obj4a12,
+    0x4a13: obj4a13,
+    0x4a1a: obj4a1a,
     0x4c01: obj4c01,
+    0x4c02: obj4c02,
+    0x4c03: obj4c03,
     0x4c08: obj4c08,
+    0x4c14: obj4c14,
+    0x4e0c: obj4e0c,
+    0x4e0d: obj4e0d,
+    0x4e0e: obj4e0e,
+    0x4e16: obj4e16,
+    0x4e17: obj4e17,
+    0x5010: obj5010,
+    0x5011: obj5011,
+    0x5403: obj5403,
+    0x5601: obj5601,
+    0x5a16: obj5a16,
 }
 
 
@@ -807,7 +1147,7 @@ def parse_xiaomi(self, data, source_mac, rssi):
 
     result = {
         "rssi": rssi,
-        "mac": ''.join(f'{i:02X}' for i in xiaomi_mac),
+        "mac": to_unformatted_mac(xiaomi_mac),
         "type": device_type,
         "packet": packet_id,
         "firmware": firmware,
@@ -832,7 +1172,7 @@ def parse_xiaomi(self, data, source_mac, rssi):
             if obj_length != 0:
                 resfunc = xiaomi_dataobject_dict.get(obj_typecode, None)
                 if resfunc:
-                    if hex(obj_typecode) in ["0x1001", "0xf", "0xb"]:
+                    if hex(obj_typecode) in ["0x8", "0x100e", "0x1001", "0xf", "0xb"]:
                         result.update(resfunc(dobject, device_type))
                     else:
                         result.update(resfunc(dobject))
@@ -921,8 +1261,3 @@ def decrypt_mibeacon_legacy(self, data, i, xiaomi_mac):
         )
         return None
     return decrypted_payload
-
-
-def to_mac(addr: int):
-    """Return formatted MAC address"""
-    return ':'.join(f'{i:02X}' for i in addr)
